SystemOrganization addCategory: #'Math-Tests-Quaternion'!

TestCase subclass: #QuaternionTest
	instanceVariableNames: 'q1234 q1 q12 q123'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-Quaternion'!

!QuaternionTest commentStamp: '<historical>' prior: 0!
This class is for unit testing the Quaternion.!

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
setUp
	q1234 := 1 i: 2 j: 3 k: 4.
	q1 := 1 asQuaternion.
	q12 := (1 + 2 i) asQuaternion .
	q123 := (1 + 2 i + 3 j).! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testAbs
	self assert: (1 i: 1 j: 1 k: 1) abs = 2.
	self should: [(1.0e200 i: 1.0e200 j: 1.0e200 k: 1.0e200) abs = 2.0e200]
		description: 'An overflow should be avoided'.
	self 
		should: 
			[| big |
			big := (2.0 raisedTo: 53) - 1 timesTwoPower: 1023 - 52.
			(big i: big j: big k: big) abs = Float infinity]
		description: 'An overflow cannot be avoided'.
	self should: [(1.0e-200 i: 1.0e-200 j: 1.0e-200 k: 1.0e-200) abs = 2.0e-200]
		description: 'An underflow should be avoided'! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testAddPolynomial	
	|  poly |
	poly :=	DhbPolynomial coefficients: #(1 1 1).
	self should: [(poly + q12 at: 0 ) = (2 + 2  i) asQuaternion ].
	self should: [(q12 + poly at: 0) = (2  + 2  i) asQuaternion ].
	! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testAngle
	self should: [ q1 angle = 0].
	self should: [(q12 - q1) angle = Float pi].
	self should: [(q123 - q12) angle = Float pi].
	self should: [(q1234 - q123) angleInDegrees = 180].! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testAsQuaternion
	self should: [1 asQuaternion = q1 asQuaternion].
	self should: [q1234 asQuaternion = q1234].
	! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testCos
	| eps |
	eps := 1.0e-6.
	self assert: (q1234 cos - ((q1234 real cos * q1234 unreal cos) - (q1234 real sin * q1234 unreal sin))) abs < eps.
	self assert: ((1 + 2 i) cos - (1 + 2 j k) cos) abs < eps.! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testCosh
	| eps |
	eps := 1.0e-6.
	self assert: ((1 + 2 i) cosh - (1 + 2 j k) cosh) abs < eps! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/31/2012 09:51'!
testDividePolynomial	
	|  poly |
	poly :=	DhbPolynomial coefficients: #(1 1 1).
	self should: [(poly / q12 at: 0 ) = q12 reciprocal ].
	self should: [(q12 / poly at: 0) = 0].
	poly := poly * q12.
	self should: [(poly / q12) coefficients allSatisfy: [:ea | ea = 1]]
	! !

!QuaternionTest methodsFor: 'running' stamp: 'nice 5/2/2014 15:17'!
testEquality
	self assert: q1234 = q1234 copy.
	self assert: (1 + 2 i) = (1 + 2 j k).
	self assert: (1 + 2 j k) = (1 + 2 i).
	self deny: (1 + 2 j) = (1 + 2 i).
	self deny: q1234 = 1.
	self assert: (1 k k) = -1.
	self assert: 1 = (-1 k k).
	self deny: q1234 = nil.! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testExp
	| eps |
	eps := 1.0e-6.
	self assert: (q1234 exp - (q1234 real exp * q1234 unreal exp)) abs < eps.
	self assert: ((1 + 2 i) exp - (1 + 2 j k) exp) abs < eps! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testFloatClass
	"not much to do here?"
	self should: [(1.5 * q1234) floatClass = 1.5 class].
	self should: [q12 floatClass = q12 reduce floatClass ].! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testFractionDoubleDispatch
	"needed for coverage of adaptToFraction:andSend:"
	| frac |

	frac := 1 / 2.
	self should: [(frac + q1234) = (q1234 + frac)].
	self should: [(frac + q1234) real isKindOf: Fraction].
	self should: [(frac + q1234) real = (3 / 2)]. ! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testFunctions
	| eps |
	eps := 1.0e-6.
	self assert: (q1234 ln exp - q1234) abs < eps! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testHash
	self assert: q1234 hash = q1234 copy hash.
	self assert: (1 + 2 i) hash = (1 + 2 j k) hash.
	self assert: (1 k k) hash = -1 hash.! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testLn
	| eps |
	eps := 1.0e-6.
	self assert: ((1 + 2 i) ln - (1 + 2 j k) ln) abs < eps! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testLog
	"this ensures that log and ln have the expected relationship"
	| qln qlg10ln |
	qln := q1234 ln.
	qlg10ln := q1234 log * 10 ln.
	
	"this inspires me to implement Quaternion>>equalsTo: "
	self should: [qln qr equalsTo: qlg10ln real].
	self should: [qln qi equalsTo: qlg10ln qi].
	self should: [qln qj equalsTo: qlg10ln qj].
	self should: [qln qk equalsTo: qlg10ln qk].! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testOne
	self assert: Quaternion one unreal isZero.
	self assert: Quaternion one real = 1.! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testPrintOn
	| s | 
	s := q1234 printString .
	self should: [s = '(1 i: 2 j: 3 k: 4)'].! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testProduct
	self  assert: (q1234) * 5 = (5 i: 10 j: 15 k: 20).
	self  assert: 5 * (q1234) = (5 i: 10 j: 15 k: 20).
	self assert: 1 i * 1 i = -1.
	self assert: 1 j * 1 j = -1.
	self assert: 1 k * 1 k = -1.
	self assert: 1 i * 1 j = 1 k.
	self assert: 1 j * 1 k = 1 i.
	self assert: 1 k * 1 i = 1 j.
	self assert: 1 j * 1 i = -1 k.
	self assert: 1 k * 1 j = -1 i.
	self assert: 1 i * 1 k = -1 j.
	self assert: 1 i i = -1.
	self assert: 1 j j = -1.
	self assert: 1 k k = -1.
	self assert: 1 i j = 1 k.
	self assert: 1 j k = 1 i.
	self assert: 1 k i = 1 j.
	self assert: 1 j i = -1 k.
	self assert: 1 k j = -1 i.
	self assert: 1 i k = -1 j.
	self  assert: (q1234) * 1 i = (q1234) i.
	self  assert: (q1234) * 1 j = (q1234) j.
	self  assert: (q1234) * 1 k = (q1234) k.

	self assert: (q1234 * 0) isZero.
	self assert: (0 / q1234) isZero.

	self assert: (q1234 * q1234 conjugated) = q1234 squaredNorm.
	self assert: (1 / q1234) = q1234 reciprocal.
	self assert: (q1234 / q1234) = 1.! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/31/2012 09:22'!
testProductWithVector
	| vec |
	vec := DhbVector new: 2.
	vec at: 1 put: 1.
	vec at: 2 put: 2.
	"This uses productWithVector"
	self should: [(vec * q1234 at: 2 )= (2 * q1234)].
	
	"This uses adaptToQuaternion:andSend:"
	self should: [(q1234 * vec at: 1) = q1234].! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testRaisedTo
	| eps |
	eps := 1.0e-6.
	self assert: ((q1234 raisedTo: 3) - (q1234 ln * 3) exp) abs < eps.! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/30/2012 15:50'!
testRandomIsLessThanOne
	| r |
	r := Quaternion random.
	self should: [r abs >= 0.0].
	self should: [r abs <= 1.0].
	self should: [r isQuaternion].
	r := q1234 random.
	self should: [r qr < q1234 abs].
	self should: [r qi < q1234 abs ].
	self should: [r qj < q1234 abs].
	self should: [r qk < q1234 abs].
	! !

!QuaternionTest methodsFor: 'running' stamp: 'nice 4/30/2014 00:21'!
testReduce
	self should: [q1234 reduce = q1234].
	self should: [q1 reduce = 1].
	self should: [q1 reduce isInteger].
	self should: [q12 reduce isComplexNumber ].
	self should: [q12 reduce = (1 + 2 i) asComplex ].
	self shouldnt: [q123 reduce isComplexNumber ].! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testSin
	| eps |
	eps := 1.0e-6.
	self assert: (q1234 sin - ((q1234 real sin * q1234 unreal cos) + (q1234 real cos * q1234 unreal sin))) abs < eps.
	self assert: ((1 + 2 i) sin - (1 + 2 j k) sin) abs < eps.! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testSinh
	| eps |
	eps := 1.0e-6.
	self assert: ((1 + 2 i) sinh - (1 + 2 j k) sinh) abs < eps! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/31/2012 07:55'!
testSubtractPolynomial	
	|  poly |
	poly :=	DhbPolynomial coefficients: #(1 1 1).
	self should: [(poly - q12 at: 0 ) = (0 - 2  i) asQuaternion ].
	self should: [(q12 - poly at: 0) = (0 + 2  i) asQuaternion ].
	! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testSum
	self  assert: 1 + 2 i + 3 j + 4 k = (q1234).
	self  assert: (1 + 2 i )+ (3 + 4 i) j = (q1234).
	self  assert: (q1234) + 5 = (6 i: 2 j: 3 k: 4).
	self  assert: 5 + (q1234) = (6 i: 2 j: 3 k: 4).
	self  assert: (q1234) - 9 = (-8 i: 2 j: 3 k: 4).
	self  assert: 9 - (q1234) = (8 i: -2 j: -3 k: -4).

	self assert: q1234 + q1234 conjugated  = (q1234 real * 2).
	self assert: q1234 + q1234 negated = (0).
	self assert: q1234 isZero not.
	self assert: (q1234 negated + q1234) isZero.! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testTan
	| eps |
	eps := 1.0e-6.
	self assert: ((1 + 2 i) tan - (1 + 2 j k) tan) abs < eps! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testTanh
	| eps |
	eps := 1.0e-6.
	self assert: ((1 + 2 i) tanh - (1 + 2 j k) tanh) abs < eps! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/31/2012 08:45'!
testTimesPolynomial	
	|  poly |
	poly :=	DhbPolynomial coefficients: #(1 1 1).
	self should: [(poly * q12 at: 0 ) = q12 ].
	self should: [(q12 * poly at: 0) = q12].
	! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testUnreal
	self assert: q1234 unreal real = 0.
	self assert: q1234 unreal squared = (2 squared + 3 squared + 4 squared) negated.! !

!QuaternionTest methodsFor: 'running' stamp: 'DanielUber 5/20/2012 18:26:23.492'!
testZero
	self assert: Quaternion zero isZero.
	self assert: Quaternion zero abs isZero.! !
